require 'active_support/core_ext/class/inheritable_attributes'


module NetAtlas::Plugin
  class Base
    SEVERITIES = [:ok, :warning, :critical, :unknown, :error]

    class_inheritable_accessor :plugin_args
    self.plugin_args = {
      :critical => {:type => Integer, :default => 10},
      :warning => {:type => Integer, :default => 5}
    }

    class << self
      # voodoo
      def plugin_arguments &block
        def (self.plugin_args).method_missing(meth, *args)
          meth = meth.to_s.gsub("=",'').to_sym
          return self[meth] unless args
          arg = args.first
          self[meth] = if arg.kind_of? Class
            case arg.to_s
            when 'Integer', 'String'
              {:type => arg, :default => nil}
            end
          else
            case arg
            when true, false
              {:type => 'Boolean', :default => arg}
            when Integer, String
              {:type => arg.class, :default => arg}
            when Array
              {:type => arg.class, :options => arg}
            end
          end
        end

        self.plugin_args.instance_eval(&block)
      end

      def arguments
        self.plugin_args
      end
    end


    def poll
      raise "Must override this method"
    end
  end

end